using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Eco.Handles;
using Eco.Windows.Forms;
using Eco.WinForm;
using Eco.ObjectRepresentation;
using Eco.Services;

using DevGroupGettingStartedWithEco.Model;


namespace DevGroupGettingStartedWithEco
{
    /// <summary>
    /// Summary description for ManageInvoices.
    /// </summary>
    public partial class ManageInvoices : System.Windows.Forms.Form
    {
        protected static class InvoiceStates
        {
            public const string Raising = "Raising";
        }

        private const string InvoiceNotEditable = "Invoice is no longer in Raising State.  Editing is not allowed";
        

        public ManageInvoices(DevGroupGettingStartedWithEco.DevGroupGettingStartedWithEcoEcoSpace ecoSpace)
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            // Set EcoSpace (actually stores it in rhRoot.EcoSpace)
            // Note that you need to set the EcoSpace property for 
            // each ReferenceHandle and VariableHandle you add to 
            // the form.
            rhRoot.EcoSpace = ecoSpace;
        }

        public DevGroupGettingStartedWithEco.DevGroupGettingStartedWithEcoEcoSpace EcoSpace
        {
            get { return (DevGroupGettingStartedWithEco.DevGroupGettingStartedWithEcoEcoSpace)rhRoot.EcoSpace; }
        }

        private void btnNewInvoice_Click(object sender, EventArgs e)
        {
            Invoice addition = new Invoice(this.EcoSpace);
            addition.Dated = DateTime.Now;
            addition.Reference = "Change this Reference Number";
        }

        private void btnAddToInvoice_Click(object sender, EventArgs e)
        {
            // Here we want to get the Currently Selected Invoice, and the Currently Selected 
            // Available Product and then if they aren't null create a new InvoiceItem and
            // assign it the appropriate Product.
            btnAddToInvoice.Enabled = false;
            try
            {
                if (cmhCurrentAvailableProduct.Element != null)
                {
                    Product curProduct = cmhCurrentAvailableProduct.Element.AsObject as Product;
                    if (curProduct != null)
                    {
                        if (cmhCurrentInvoice.Element != null)
                        {
                            Invoice curInvoice = cmhCurrentInvoice.Element.AsObject as Invoice;
                            if (curInvoice != null)
                            {
                                if (curInvoice.InvoiceState == InvoiceStates.Raising)
                                {
                                    InvoiceItem iiNew = new InvoiceItem(EcoSpace);
                                    iiNew.Product = curProduct;
                                    curInvoice.InvoiceItems.Add(iiNew);
                                }
                                else
                                    MessageBox.Show(InvoiceNotEditable);
                            }
                        }
                    }
                }
            }
            finally
            {
                btnAddToInvoice.Enabled = true;
            }
        }

        private void btnRemoveFromInvoice_Click(object sender, EventArgs e)
        {
            btnRemoveFromInvoice.Enabled = false;
            try
            {
                // Remove the currently selected InvoiceItem, so that it free's up the
                // associated product.
                if (cmhCurrentInvoiceItem.Element != null)
                {
                    InvoiceItem curII = cmhCurrentInvoiceItem.Element.AsObject as InvoiceItem;
                    if (curII != null)
                    {
                        if (curII.Invoice.InvoiceState == InvoiceStates.Raising)
                        {
                            curII.Product = null;
                            curII.AsIObject().Delete();
                        }
                        else
                            MessageBox.Show(InvoiceNotEditable);
                    }
                }
            }
            finally
            {
                btnRemoveFromInvoice.Enabled = true;
            }
        }

        private void btnDeleteInvoice_Click(object sender, EventArgs e)
        {
            // Get the Currently Selected Invoice, and ask the user to confirm the deletion.
            if (cmhCurrentInvoice.Element != null)
            {
                Invoice curInvioce = cmhCurrentInvoice.Element.AsObject as Invoice;
                if (curInvioce != null)
                {
                    DialogResult dr = MessageBox.Show(string.Format("Delete Invoice Reference: {0}", curInvioce.Reference), "Confirm Current Invoice Deletion", MessageBoxButtons.YesNo);
                    if (dr == DialogResult.Yes)
                    {
                        curInvioce.AsIObject().Delete();
                    }
                }
            }
        }

        private void btnSubmit_Click(object sender, EventArgs e)
        {
            Invoice currentInvoice = cmhCurrentInvoice.Element.AsObject as Invoice;
            if (currentInvoice != null)
                currentInvoice.Submit();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Invoice currentInvoice = cmhCurrentInvoice.Element.AsObject as Invoice;
            if (currentInvoice != null)
                currentInvoice.Cancel();
        }

        private void btnDispatch_Click(object sender, EventArgs e)
        {
            Invoice currentInvoice = cmhCurrentInvoice.Element.AsObject as Invoice;
            if (currentInvoice != null)
                currentInvoice.Dispatch();
        }

        private void btnReviewed_Click(object sender, EventArgs e)
        {
            Invoice currentInvoice = cmhCurrentInvoice.Element.AsObject as Invoice;
            if (currentInvoice != null)
                currentInvoice.Reviewed();
        }
    }
}